precision highp float;

#define FEATURE_TEXTURED
#define FEATURE_GRADIENT
#define FEATURE_GRADIENT_4_STOPS

#ifdef GRADIENT_4_STOPS
#define GRADIENT
#endif
#ifdef GRADIENT
#define USE_MATERIAL
#endif
#ifdef TEXTURED
#define USE_MATERIAL
#endif

#ifndef PINK
#define PINK vec4(1.0, 0.0, 1.0, 1.0)
#endif

#define USE_NDC_COORDINATES
in highp vec2 ndcCoordinates;

out lowp vec4 outColor;

uniform highp mat4 inverseProjectionMatrix;
uniform highp vec4 viewToWorld[2];
uniform lowp uint material;

#include "lib/Quaternion.glsl"
#include "lib/Textures.frag"

#ifdef USE_MATERIAL

/**
 * Material definition
 *
 * Contains texture and gradient data.
 */
struct Material {
#ifdef GRADIENT_4_STOPS
    lowp vec4 colorStop3;
    lowp vec4 colorStop2;
#endif
#ifdef GRADIENT
    lowp vec4 colorStop1;
    lowp vec4 colorStop0;
#endif
#ifdef TEXTURED
    mediump uint texture;
#endif
};

#include "lib/Materials.frag"

Material decodeMaterial(uint matIndex) {
    {{decoder}}
    return mat;
}

#endif

#include "lib/Math.glsl"
#include "lib/CoordinateSystems.glsl"
#include "lib/Color.glsl"

#ifdef TEXTURED
vec4 shadeTexture(const Material mat, vec3 direction) {
    if(mat.texture == 0u) return PINK;
    vec2 uv = cartesianToEquirectangular(direction);
    return textureAtlas(mat.texture, uv);
}
#endif

#ifdef GRADIENT
vec4 shadeGradient(const Material mat, float value) {
    #ifdef GRADIENT_4_STOPS
    return gradient4(mat.colorStop0, mat.colorStop1, mat.colorStop2, mat.colorStop3, value);
    #else
    return mix(mat.colorStop0, mat.colorStop1, value);
    #endif
}
#endif

void main() {
    #ifdef USE_MATERIAL
    Material mat = decodeMaterial(material);

    vec3 unprojPoint = (inverseProjectionMatrix*vec4(ndcCoordinates, 0.0, 1.0)).xyz;
    vec3 direction = normalize(quat_transformVector(viewToWorld[0], unprojPoint));

    outColor = vec4(1.0);

    #ifdef TEXTURED
    outColor *= shadeTexture(mat, direction);
    #endif
    #ifdef GRADIENT
    /* Remap direction to [0, 1] and sample gradient */
    outColor *= shadeGradient(mat, direction.y * 0.5 + 0.5);
    #endif

    #else

    /* Use error color if the shader isn't set for textures
     * nor for gradients */
    outColor = Pink;

    #endif
}
